package edu.mit.harold.icollections.tests;

import java.util.HashMap;
import java.util.Map;
import java.util.Random;
import java.util.Map.Entry;

import edu.mit.harold.icollections.IMap;
import edu.mit.harold.icollections.IntTreeIMap;


import junit.framework.TestCase;

public class IntTreeIMapTest extends TestCase {

	/**
	 * Compares the behavior of java.util.HashMap to the behavior of IntTreeIMap.
	 */
	public void testRandomlyAgainstJavaMap() {
		IMap<Integer,Integer> imap = IntTreeIMap.empty();
		Map<Integer,Integer> map = new HashMap<Integer,Integer>();
		Random r = new Random();
		for(int i=0;i<10000;i++) {
			if(imap.size()==0 || r.nextBoolean()) { // add
				int k = r.nextInt(), v = r.nextInt();

				assertEquals(map.containsKey(k), imap.containsKey(k));
				assertEquals(map.get(k), imap.get(k));

				map.put(k, v);
				imap = imap.plus(k, v);
			} else { // remove a random key
				int j = r.nextInt(imap.size());
				for(Entry<Integer,Integer> e : imap.entrySet()) {
					int k = e.getKey();

					assertTrue(map.containsKey(k));
					assertTrue(imap.containsKey(k));
					assertEquals(map.get(k), imap.get(k));
					assertTrue(map.entrySet().contains(e));
					assertTrue(imap.entrySet().contains(e));
					UtilityTest.assertEqualsAndHash( imap, imap.plus(k,e.getValue()) );

					if(j--==0) {
						map.remove(k);
						imap = imap.minus(k);
						assertFalse(imap.entrySet().contains(e));
					}
				}
			}
			
			// also try to remove a _totally_ random key:
			int k = r.nextInt();
			assertEquals(map.containsKey(k), imap.containsKey(k));
			assertEquals(map.get(k), imap.get(k));
			map.remove(k);
			imap = imap.minus(k);
			
			// and try out a non-Integer:
			String s = Integer.toString(k);
			assertFalse( imap.containsKey(s) );
			assertEquals(null, imap.get(s));
			assertFalse( imap.entrySet().contains(s) );
			imap = imap.minus(s);

			assertEquals(map.size(), imap.size());
			UtilityTest.assertEqualsAndHash(map, imap);
			UtilityTest.assertEqualsAndHash(map.entrySet(), imap.entrySet());

			UtilityTest.assertEqualsAndHash(imap, IntTreeIMap.from(imap));
			UtilityTest.assertEqualsAndHash(IntTreeIMap.empty(), imap.minusAll(imap.keySet()));
			UtilityTest.assertEqualsAndHash(imap, imap.plusAll(imap));
		}
	}

	public void testEmpty() {
		IMap<?,?> empty = IntTreeIMap.empty();
		UtilityTest.assertEqualsAndHash(new HashMap<Object,Object>(), empty);
		assertEquals(0, empty.size());
		assertTrue(empty.isEmpty());
		for(@SuppressWarnings("unused") Object e : empty.entrySet())
			fail();
	}

	public void testSingleton() {
		UtilityTest.assertEqualsAndHash(IntTreeIMap.empty().plus(10, "test"),
				IntTreeIMap.singleton(10, "test"));
	}
	
	public void testWithKeysChanged() {
		// TODO
	}

}
